from ._calculate_q_eq.match_closest_pair import match_closest_pairs
from ._calculate_q_eq.create_count_matrices import create_count_matrices
from ._calculate_q_eq.calculate_q_eq import calculate_q_eq

# COMPARE_INDELS_FLAG decides if indels should be included when comparing likeness of sequences
def bw_estimator(FORMAT, THRESHOLD, MULTIALIGNMENT_LIST, COMPARE_INDELS_FLAG = False):
    aggregated_count_matrix_list = []
    
    for MULTIALIGNMENT in MULTIALIGNMENT_LIST:
        CLOSEST_PAIRS = match_closest_pairs(MULTIALIGNMENT, COMPARE_INDELS_FLAG)
        COUNT_MATRIX_LIST = create_count_matrices(CLOSEST_PAIRS)
        aggregated_count_matrix_list.extend(COUNT_MATRIX_LIST)

    Q, EQ = calculate_q_eq(aggregated_count_matrix_list, THRESHOLD)
    
    return Q, EQ


import numpy as np
from Bio import SeqIO,AlignIO

def handle_input_file(FILE_PATH, FORMAT):
    sequence_list = SeqIO.parse(FILE_PATH, "fasta")
    sequence_list = [sequence.seq._data for sequence in sequence_list]
    
    for index, sequence in enumerate(sequence_list):
        sequence_list[index] = np.array(list(sequence))
        
    return sequence_list


import setuptools

with open("README.txt", "r") as fh:
    long_description = fh.read()
	
setuptools.setup(
	name='modelestimator_v2',
	version='0.0.1',
	author='Ruben Ridderström',
	author_email='ruben.ridderstrom@gmail.com',
	description='Program for estimating amino acid replacement rates',
	long_description=long_description,
	long_description_content_type="text/plain; charset=UTF-8",
	url='https://github.com/RubenRidderstrom/modelestimator',
	license='GPLv3',
	packages = setuptools.find_packages(),
	entry_points={
		'console_scripts':[
			'modelestimator = modelestimator.main:main'
		]
	},
	install_requires=[
		'numpy',
		'scipy',
		'biopython',
	],
	setup_requires=['pytest-runner'],
	tests_require=['pytest']
	)


import numpy as np

### Private functions
def _matching_letters(a,b, COMPARE_INDELS_FLAG):
    assert(len(a) == len(b))

    number_of_matching_positions = 0

    if COMPARE_INDELS_FLAG:
        for x,y in zip(a,b):
            if x == y and x != '-' :
                number_of_matching_positions += 1
    else:
        number_of_matching_positions = np.sum(a==b) # Faster than when indels have to be ignored
                   
    return number_of_matching_positions

### Interface
def match_closest_pairs(sequence_list, COMPARE_INDELS_FLAG):
    assert (len(sequence_list) % 2) == 0

    indexes_and_matching_letters = []

    for PRIMARY_INDEX, PRIMARY_SEQUENCE in enumerate(sequence_list):
        for SECONDARY_INDEX in range(PRIMARY_INDEX+1, len(sequence_list)):
            SECONDARY_SEQUENCE = sequence_list[SECONDARY_INDEX]
            MATCHING_LETTERS = _matching_letters(PRIMARY_SEQUENCE, SECONDARY_SEQUENCE, COMPARE_INDELS_FLAG)

            INDEXES = (PRIMARY_INDEX, SECONDARY_INDEX)
            INDEX_SCORE_TUPLE = (INDEXES, MATCHING_LETTERS)
            indexes_and_matching_letters.append(INDEX_SCORE_TUPLE)

    indexes_and_matching_letters.sort(key=lambda tup: tup[1], reverse = True)   # Sort on matching letters
    matched_indexes = []
    closest_pairs = []
    NUMBER_OF_SEQUENCES = len(sequence_list)

    while len(matched_indexes) != NUMBER_OF_SEQUENCES:
        CURRENT_INDEX_AND_MATCHING_LETTERS_TUPLE = indexes_and_matching_letters.pop(0)
        FIRST_INDEX = CURRENT_INDEX_AND_MATCHING_LETTERS_TUPLE[0][1]
        SECOND_INDEX = CURRENT_INDEX_AND_MATCHING_LETTERS_TUPLE[0][0]

        if not(FIRST_INDEX in matched_indexes) and not(SECOND_INDEX in matched_indexes):
            matched_indexes.append(FIRST_INDEX)
            matched_indexes.append(SECOND_INDEX)

            FIRST_SEQUENCE = sequence_list[FIRST_INDEX]
            SECOND_SEQUENCE = sequence_list[SECOND_INDEX]
            closest_pairs.append((FIRST_SEQUENCE, SECOND_SEQUENCE))

    return closest_pairs